import { app } from '@/app.ts';
import { appPathCheck, installApp } from './module/install-app.ts';
import { manager } from './manager-app.ts';
import { selfRestart } from '@/modules/self-restart.ts';
import { AppListModel } from '../app-manager/module/index.ts';
// curl http://localhost:4002/api/router?path=micro-app&key=deploy
// 把对应的应用安装到系统的apps目录下，并解压，然后把配置项写入数据库配置
// key 是应用的唯一标识，和package.json中的key一致，绑定关系
// 安装就就一定存在状态。这个时候需要判断这个应用的依赖关系，这个应用的开启状态。
// 修改app的类型，这些模块是存在添加，不存在则删除。
app
  .route({
    path: 'micro-app',
    key: 'deploy',
    description: 'Deploy micro app in server',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const data = ctx.query?.data;
    const { id, key, force, install } = data;
    if (!id) {
      ctx.throw(400, 'Invalid id');
    }
    let username = tokenUser.username;
    if (data.username) {
      // username = data.username;
    }
    const microApp = await AppListModel.findByPk(id);
    if (!microApp) {
      ctx.throw(400, 'Invalid id');
    }
    const { key: appKey, version } = microApp;
    const check = await appPathCheck({ key: key });
    let appType: string;
    if (check) {
      if (!force) {
        ctx.throw(400, 'App already exists, please remove it first');
      } else {
        const app = manager.getAppShowInfo(key);
        appType = app?.type;

        await manager.removeApp(key);
      }
    }
    const installAppData = await installApp({ username, appKey, version, key, needInstallDeps: !!install });
    await manager.add(installAppData.showAppInfo);
    ctx.body = installAppData;
    if (appType === 'system-app') {
      // 如果是system-app(主进程运行），就重启服务
      setTimeout(() => selfRestart(), 3000);
    }
  })
  .addTo(app);

//  curl http://localhost:4002/api/router?path=micro-app&key=unload
app
  .route({
    path: 'micro-app',
    key: 'unload',
    description: 'Unload micro app in server',
  })
  .define(async (ctx) => {
    const { key } = ctx.query?.data;
    // const key = 'mark';
    const main = manager.getAppShowInfo(key);
    if (!main) {
      ctx.throw(400, 'Invalid app');
    }
    manager.removeApp(key);
    ctx.body = main;
  })
  .addTo(app);
